<?php
/**
 * Classe PDFMerger
 * Esta classe é responsável por unir vários arquivos PDF em um único arquivo.
 * 
 * Autor: Rafael Pickbrenner
 * 
 * A classe **PDFMerger** é usada para unir vários arquivos PDF em um único arquivo. Ela tem métodos para adicionar
 * arquivos PDF individuais ou vários de uma vez, unir os arquivos PDF adicionados em um único arquivo e excluir os
 * arquivos de entrada e saída.
 * 
 * Além disso, a classe agora inclui um método para baixar um arquivo PDF de uma URL e adicioná-lo à lista de arquivos
 * a serem unidos. Este método baixa o arquivo PDF, salva-o no diretório raiz e o adiciona à lista.
 * 
 * Para usar a classe:
 * 1. Instanciá-la com o diretório raiz onde os arquivos PDF estão localizados.
 * 2. Adicionar arquivos PDF à lista de arquivos a serem unidos usando os métodos **adicionarPDF**,
 *    **adicionarPDFPorURL** e/ou **adicionarVariosPDFs**.
 * 3. Unir os arquivos PDF em um único arquivo usando o método **unirPDFs**.
 * 4. Se necessário excluir os arquivos de entrada e saída usando os métodos **excluirArquivos** e
 *    **excluirArquivoSaida**, respectivamente.
 * 
 * Exemplo de uso:
 * $merger = new PDFMerger( $this->hcmParameters->getTempDirectory() );
 * 
 * $merger->adicionarPDF( 'arquivo_pdf_1.pdf' );
 * $merger->adicionarPDF( 'arquivo_pdf_2.pdf' );
 * $merger->adicionarPDF( 'arquivo_pdf_3.pdf' );
 * //ou
 * $merger->adicionarVariosPDFs( array( 'arquivo_pdf_1.pdf', 'arquivo_pdf_2.pdf', 'arquivo_pdf_3.pdf' ) );
 * //ou
 * $merger->adicionarPDFPorURL( 'http://exemplo.com/arquivo_pdf.pdf', 'arquivo_pdf_4.pdf' );
 * 
 * $caminhoArquivoMesclado = $merger->unirPDFs( 'arquivo_mesclado.pdf' );
 * 
 * $merger->excluirArquivos();
 * $merger->excluirArquivoSaida();
 */

namespace HCM\Geral\Logic;

class PDFMerger {
    const ERRO_ARQUIVO_NAO_ENCONTRADO = 'Erro ao adicionar arquivo PDF: %s não encontrado.';
    const ERRO_ARQUIVO_NAO_ADICIONADO = 'Erro ao unir PDFs: nenhum arquivo PDF foi adicionado.';
    const ERRO_BAIXAR_ARQUIVO = 'Erro ao baixar o arquivo PDF da URL: %s';
    const ERRO_SALVAR_ARQUIVO = 'Erro ao salvar o arquivo PDF baixado no diretório raiz: %s';
    const ERRO_EXECUCAO_PDFTK = 'Erro ao executar o PDFtk: %s';
    const PDFTK_PATH = '/usr/bin/pdftk';

    private $diretorioRaiz;
    private $arquivoGerado;
    private $arquivosPDF = [];

    public function __construct($diretorioRaiz = '/tmp/') {
        $this->diretorioRaiz = rtrim($diretorioRaiz, '/');
    }

    public function adicionarPDF($nomeArquivoPDF) {
        $arquivoPDF = $this->diretorioRaiz . '/' . $nomeArquivoPDF;
        if (file_exists($arquivoPDF)) {
            $this->arquivosPDF[] = $arquivoPDF;
        } else {
            throw new \Exception(sprintf(self::ERRO_ARQUIVO_NAO_ENCONTRADO, $nomeArquivoPDF));
        }
    }

    public function adicionarVariosPDFs($nomesArquivosPDF) {
        foreach ($nomesArquivosPDF as $nomeArquivoPDF) {
            $this->adicionarPDF($nomeArquivoPDF);
        }
    }

    public function adicionarPDFPorURL($url, $nomeArquivoPDF) {
        $arquivoPDF = $this->diretorioRaiz . '/' . $nomeArquivoPDF;

        // Verifica se o arquivo já existe e o exclui se existir
        if (file_exists($arquivoPDF)) {
            unlink($arquivoPDF);
        }
        // Baixa o arquivo PDF
        $conteudoPDF = file_get_contents( $url );
        if ($conteudoPDF === false) {
            throw new \Exception( sprintf( self::ERRO_BAIXAR_ARQUIVO, $url ) );
        }
        //$conteudoPDF = str_replace( '%PDF-1.5', '%PDF-1.3', $conteudoPDF );

        // Salva o arquivo PDF no diretório raiz
        $resultado = file_put_contents( $arquivoPDF, $conteudoPDF );
        if ($resultado === false) {
            throw new \Exception( sprintf( self::ERRO_SALVAR_ARQUIVO, $arquivoPDF ) );
        }
        chmod( $arquivoPDF, 0755 );

        // Adiciona o arquivo PDF à lista de arquivos a serem unidos, todas as páginas serão unidas
        $this->arquivosPDF[] = $arquivoPDF;
    }

    public function unirPDFs($nomeArquivoSaida) {
        if (empty($this->arquivosPDF)) {
            throw new \Exception(self::ERRO_ARQUIVO_NAO_ADICIONADO);
        }

        $command = self::PDFTK_PATH . ' ';
        $this->arquivoGerado = $this->diretorioRaiz . '/' . $nomeArquivoSaida . '.pdf';

        foreach ($this->arquivosPDF as $file) {
            $command .= '"' . $file . '" '; // Coloca os nomes dos arquivos entre aspas para lidar com espaços
        }

        //$command .= 'cat output "' . $this->arquivoGerado . '"';
        $command .= 'cat output "' . $this->arquivoGerado . '" 2>&1'; // Redireciona stderr para stdout
        exec($command, $output, $returnCode);

        if ($returnCode !== 0) {
            $errorOutput = implode("\n", $output);
            $this->arquivoGerado = null;
            throw new \Exception(sprintf(self::ERRO_EXECUCAO_PDFTK, $errorOutput));
        }

        return $this->arquivoGerado;
    }

    public function excluirArquivos() {
        foreach ($this->arquivosPDF as $arquivo) {
            if (file_exists($arquivo['path'])) {
                unlink($arquivo['path']);
            }
        }
        $this->arquivosPDF = [];
    }

    public function excluirArquivoSaida() {
        if ($this->arquivoGerado && file_exists($this->arquivoGerado)) {
            unlink($this->arquivoGerado);
        }
        $this->arquivoGerado = null; // Garante que a variável seja limpa
    }
}