<?php

namespace HCM\Geral\Logic;

use ZipArchive;

class zipReport
{
    private $diretorioTemporario; // Diretório para arquivos temporários
    private $arquivosParaZipar = []; // Lista de arquivos a serem zipados

    public function __construct(string $diretorioTemporario = null)
    {
        // Define o diretório temporário. Se não for fornecido, usa o padrão do sistema.
        $this->diretorioTemporario = $diretorioTemporario ?? sys_get_temp_dir() . '/zipReport_temp/';
        if (!is_dir($this->diretorioTemporario)) {
            mkdir($this->diretorioTemporario, 0777, true); // Cria diretório se não existir
        }
    }

    public function adicionarArquivoUrl(string $url, string $nomeArquivo, string $extensao, array $parametrosUrl = []): bool
    {
        $nomeArquivoCompleto = $this->gerarNomeUnicoArquivo($nomeArquivo . '.' . $extensao);
        $caminhoSalvar = $this->diretorioTemporario . $nomeArquivoCompleto;

        if (!$this->baixarArquivoUrl($url, $caminhoSalvar, $parametrosUrl)) {
            return false; // Falha ao baixar o arquivo
        }

        $this->arquivosParaZipar[] = [
            'caminho' => $caminhoSalvar,
            'nomeNoZip' => $nomeArquivo . '.' . $extensao, // Mantém a extensão original no ZIP
        ];
        return true;
    }

    public function adicionarArquivoLocal(string $caminhoArquivo, string $nomeArquivoNoZip = null): bool
    {
        if (!file_exists($caminhoArquivo) || !is_readable($caminhoArquivo)) {
            return false; // Arquivo local não existe ou não é legível
        }

        $nomeFinalNoZip = $nomeArquivoNoZip ?? basename($caminhoArquivo); // Usa o nome base se não for especificado

        $this->arquivosParaZipar[] = [
            'caminho' => $caminhoArquivo,
            'nomeNoZip' => $nomeFinalNoZip,
        ];
        return true;
    }


    public function compactarParaZip(string $nomeZipArquivo): ?string
    {
        if (empty($this->arquivosParaZipar)) {
            return null; // Nada para compactar
        }

        $caminhoZipCompleto = $this->diretorioTemporario . $nomeZipArquivo;
        $zip = new ZipArchive();

        if ($zip->open($caminhoZipCompleto, ZipArchive::CREATE | ZipArchive::OVERWRITE) !== TRUE) {
            return null; // Falha ao criar o arquivo ZIP
        }

        foreach ($this->arquivosParaZipar as $arquivoInfo) {
            if (file_exists($arquivoInfo['caminho'])) {
                $zip->addFile($arquivoInfo['caminho'], $arquivoInfo['nomeNoZip']);
            }
        }

        $zip->close();
        $this->limparArquivosTemporarios(); // Limpa arquivos temporários após a compactação
        return $caminhoZipCompleto;
    }

    private function baixarArquivoUrl(string $url, string $caminhoSalvar, array $parametrosUrl = []): bool
    {
        $urlCompleta = $url;
        if (!empty($parametrosUrl)) {
            $urlCompleta = $url . '?' . http_build_query($parametrosUrl); // Adiciona parâmetros à URL se houver
        }

        $conteudoArquivo = @file_get_contents($urlCompleta); // Use @ para suprimir warnings em caso de erro HTTP

        if ($conteudoArquivo === false) {
            return false; // Falha ao baixar o conteúdo da URL
        }

        if (file_put_contents($caminhoSalvar, $conteudoArquivo) === false) {
            return false; // Falha ao salvar o conteúdo no arquivo
        }

        return true;
    }

    private function limparArquivosTemporarios(): void
    {
        foreach ($this->arquivosParaZipar as $arquivoInfo) {
            if (file_exists($arquivoInfo['caminho'])) {
                unlink($arquivoInfo['caminho']); // Apaga o arquivo temporário
            }
        }
        $this->arquivosParaZipar = []; // Limpa a lista de arquivos
    }

    private function gerarNomeUnicoArquivo(string $nomeBase): string
    {
        return pathinfo($nomeBase, PATHINFO_FILENAME) . '_' . uniqid() . '.' . pathinfo($nomeBase, PATHINFO_EXTENSION);
    }
}