<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;


class RelatoriosHcmRepository {

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    /**
     * Obtém as estruturas legal, gerencial e superior dos vínculos ativos na competência informada.
     *
     * A função retorna uma lista de vínculos e suas respectivas estruturas, considerando apenas os vínculos que
     * possuem líquido de pagamento (mensal, parcial ou de rescisão) na competência especificada.
     *
     * A busca pelos vínculos é feita com base nos seguintes critérios:
     *  - Tipo de movimento
     *  - Modalidade de cálculo
     *  - Número de ocorrência de cálculo
     *  - Inclusão de 13º salário
     *  - Ignorar ocorrências de cálculo
     *
     * @param string|int $nrorg Número da organização.
     * @param string $dtmescompetenc Data da competência.
     * @param string|int $estruturalegal Estrutura legal.
     * @param string|int $nrestrutsup Estrutura superior.
     * @param string|int $nrtipomoviment Tipo de movimento.
     * @param string|int $nrtpmodalidcal Tipo de modalidade de cálculo.
     * @param string|int $nrtipohierarqorg Tipo de hierarquia organizacional.
     * @param string|int $nrocorrecal Número de ocorrência de cálculo.
     * @param string $incluir13 Indicador se deve incluir o 13º salário ('S' ou 'N').
     * @param string $ignoraocorr Indicador se deve ignorar ocorrências de cálculo ('S' ou 'N').
     *
     * @return array Um array contendo as estruturas legal, gerencial e superior do vínculo.
     */
    public function getLegSupGer($nrorg, $dtmescompetenc, $estruturalegal, $nrestrutsup, $nrtipomoviment, $nrtpmodalidcal, $nrtipohierarqorg, $nrocorrecal, $incluir13, $ignoraocorr) {
        $query = <<<SQL
SELECT
    vh.nrestrutlegal,
    H.nrestruturamsup,
    eh.nmestruturah nmestruturamsup
FROM gpe_vinculom vm
JOIN gpe_vinculoh vh ON vh.nrorg          = vm.nrorg
                    AND vh.nrvinculom     = vm.nrvinculom
                    AND vh.dtmescompetenc = ( SELECT dtmescompetenc FROM ( SELECT dtmescompetenc FROM gpe_vinculoh
                                              WHERE dtmescompetenc <= :p_dtmescompetenc
                                                AND nrorg           = vm.nrorg
                                                AND nrvinculom      = vm.nrvinculom
                                              ORDER BY dtmescompetenc DESC ) WHERE ROWNUM = 1 )
JOIN hierarquiaorg H ON H.nrorg            = vm.nrorg 
                    AND H.nrestruturamrel IN ( vh.nrestrutlegal, vh.nrestrutgeren )
                    AND H.nrtipohierarqorg = :p_nrtipohierarqorg
                    AND H.dtinivigencia    = ( SELECT dtinivigencia FROM ( SELECT dtinivigencia
                                               FROM hierarquiaorg
                                               WHERE nrorg            = H.nrorg
                                                 AND nrtipohierarqorg = H.nrtipohierarqorg
                                                 AND nrestruturamrel  = H.nrestruturamrel
                                                 AND dtinivigencia   <= :p_dtmescompetenc
                                               ORDER BY dtinivigencia DESC ) WHERE ROWNUM = 1 )
LEFT JOIN estruturah eh ON eh.nrorg          = vm.nrorg
                       AND eh.nrestruturam   = H.nrestruturamsup
                       AND eh.dtmescompetenc = ( SELECT dtmescompetenc FROM ( SELECT dtmescompetenc
                                                 FROM estruturah
                                                 WHERE nrorg        = eh.nrorg
                                                   AND nrestruturam = H.nrestruturamsup
                                                   AND dtmescompetenc <= :p_dtmescompetenc
                                                 ORDER BY dtmescompetenc DESC ) WHERE ROWNUM = 1 )
WHERE vm.nrorg = :p_nrorg
  AND ( vm.dtrescisaovinc IS NULL OR vm.dtrescisaovinc > :p_dtmescompetenc )
  AND ( instr( ',' || :p_estruturalegal || ',', ',' || vh.nrestrutlegal  || ',' ) > 0 OR :p_estruturalegal IS NULL )
  AND ( instr( ',' || :p_nrestrutsup    || ',', ',' || H.nrestruturamsup || ',' ) > 0 OR :p_nrestrutsup    IS NULL )
  AND EXISTS ( SELECT 1
               FROM fpa_itecalcfolha icf
               JOIN fpa_calculofolha cf ON cf.nrorg          = icf.nrorg
                                       AND cf.nrcalculofolha = icf.nrcalculofolha
                                       AND cf.nrtipomoviment IN ( DECODE( :p_incluir13, 'S', 4, 0 ), :p_nrtipomoviment )
                                       AND ( cf.nrocorrecal = :p_nrocorrecal OR :p_ignoraocorr = 'S')
                                       AND cf.nrtpmodalidcal = :p_nrtpmodalidcal
                                       AND cf.dtocorrencia   = :p_dtmescompetenc
               WHERE icf.nrorg      = vm.nrorg
                 AND icf.nrvinculom = vh.nrvinculom
                 AND icf.nreventom IN ( 3030, 3040, 3050 ) )
GROUP BY vh.nrestrutlegal, H.nrestruturamsup, eh.nmestruturah
ORDER BY vh.nrestrutlegal, eh.nmestruturah
SQL;
        $params = array (
            "p_nrorg"            => $nrorg,
            "p_dtmescompetenc"   => $dtmescompetenc,
            "p_estruturalegal"   => $estruturalegal,
            "p_nrestrutsup"      => $nrestrutsup,
            "p_nrtipomoviment"   => $nrtipomoviment,
            "p_nrtpmodalidcal"   => $nrtpmodalidcal,
            "p_nrtipohierarqorg" => $nrtipohierarqorg,
            "p_nrocorrecal"      => $nrocorrecal,
            "p_incluir13"        => $incluir13,
            "p_ignoraocorr"      => $ignoraocorr
        );
        
        return $this->connection->fetchAll( $query, $params );
    }

} 
